<?xml version='1.0'?> 

<!--
  Filename:	Do178Builder.xsl
  Purpose:	Customization layer for DocBook XSL stylesheets,
		suitable for use with Do178Builder.
  Mod History:	03/10/2008 RSB	Created modified stylesheets 
				from the ones that came with the
				XMLmind XSL-FO converter program
				("xslutility").
		03/11/2008 RSB	Figured out that that was a poor
				approach, and created this 
				customization layer instead.  Also:
				changed page headers to eliminate
				word-wrap of the document name, and
				put the missing corporate name onto
				the title page.  The changes from 
                                the default XMLmind stylesheets are 
                                less complex than they look, but you
                                can only replace complete XSL objects
                                in the customization layer, and some 
                                of the objects replaced were pretty 
                                big even if the actual changes to 
                                them were trivial.
                03/12/2008 RSB  Made various minor adjustments that
                                won't concern anyone.  Added some 
                                general-purpose tags as DocBook
                                extensions:  <?hard-pagebreak?>,
                                <long-ulink>, and <long-string>.

  //////////////////////////////////////////////////////////////////////////
  
  Copyright by the original authors of the DocBook stylesheets.
  Customizations copyright 2008 by Ronald S. Burkey

  This file is licensed identically to the DocBook stylesheets.  You'd 
  better look at the README file for those, in order to determine your
  rights.
  
  //////////////////////////////////////////////////////////////////////////
  
  This is an XSL-stylesheet customization which could be used with the 
  XMLmind XSL-FO converter program to convert DocBook XML to PDF, RTF, OOXML,
  ODT, etc. and get something not unlike what you would get running the 
  normal Linux DocBook converter utilities (db2pdf, db2rtf, and so on),
  *without* having to directly edit any of the stylesheets that come with 
  XMLmind.  I say "not  unlike" rather than "the same", because I don't 
  understand XSL well enough yet to get it to come out exactly the same as 
  with DSSSL stylesheets.  
  
  To *use* this customization, there are two steps:
  
  1.  Edit the xsl:import line near the head of the file to point to the 
      docbook.xsl file that came with the XMLmind XSL-FO Converter program.
      For Windows users, this has already been done, assuming that you've
      installed XMLmind at the default location.  For Linux users, there's
      no obvious default location to install the XMLmind Converter, so I've
      provided a sample path that you should be able to edit in an obvious way.
      (Actually, I've just set it the same as on my own Linux computer.)
      Because paths seem to be relative to the location of the DocBook XML
      file you're converting, the path to docbook.xsl probably needs to be
      an absolute path.
  
  2.  When running the XMLmind Converter, point it at Do178Builder.xsl rather
      than at the docbook.xsl that came with the Converter.  You do this by 
      clicking on the 'EDIT' button in the Converter's user interface.
      
  If you want to make your *own* customizations, then what you do is this:
  
  A.  Create a new XSL file (say, MyCustomizations.xsl), and put the 
      following code into it.  (You'll probably have to change the path
      to Do178Builder.xml in the xsl:import line.)
      
      <?xml version='1.0'?> 
      <xsl:stylesheet  
        xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
        xmlns:fo="http://www.w3.org/1999/XSL/Format"
        version="1.0"> 

        ... Put an absolute path to Do178Builder.xsl below ...
        <xsl:import href="Do178Builder.xsl"/> 

        ... Any amount of XSL customization code you like goes here.
        Simply replace any XSL objects that already appear in 
        Do178Builder.xsl or in the stylesheets that came with XMLmind 
        (note that they're all in the same directory as docbook.xsl). ...

      </xsl:stylesheet>  
      
  B.  When you run XMLmind, choose MyCustomizations.xsl as the stylesheet
      rather than Do178Builder.xsl or docbook.xsl.

-->

<xsl:stylesheet  
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    xmlns:fo="http://www.w3.org/1999/XSL/Format"
    version="1.0"> 

<!-- The following must point to the docbook.xml that came with
     the XMLmind XSL-FO converter program. Obviously, your path
     may not be the same as mine.  The problem is that the path
     appears to be relative (when using XMLmind) to the directory
     in which the DocBook file you're converting is stored, so 
     there's really little choice other than to use an absolute
     path (or a catalog) to docbook.xsl.  Yuk!  -->
<!-- Typical Linux ... -->
<xsl:import href="/home/rburkey/xfc-42p1/xslu/config/docbook/xsl/fo/docbook.xsl"/>
<!-- Typical Windows ... -->
<!-- <xsl:import href="file:///c:/Program Files/XMLmind XSL-FO Converter/xslu/config/docbook/xsl/fo/docbook.xsl"/> -->

<!-- /////////////////////////////////////////////////////////// -->
<!-- Everything between this line and the matching line at the 
     bottom of the file replaces items with similar names from
     the standard docbook.xsl file. -->

<!-- ============== Replaces stuff from fo/autotoc.xsl. =========== -->
     
<!-- Adds a page break after the table of contents. -->
<xsl:template name="component.toc.separator">
  <!-- Customize to output something between
       component.toc and first output -->
  <fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format" break-after="page"/>
</xsl:template>

<!-- Gets rid of a warning message associated with generation
     of the table of contents. -->
<xsl:template name="toc.line">
  <xsl:variable name="id">
    <xsl:call-template name="object.id"/>
  </xsl:variable>

  <xsl:variable name="label">
    <xsl:apply-templates select="." mode="label.markup"/>
  </xsl:variable>

  <fo:block xsl:use-attribute-sets="toc.line.properties"
            end-indent="{$toc.indent.width}pt"
            last-line-end-indent="-{$toc.indent.width}pt">
    <fo:inline keep-with-next.within-line="always">
      <fo:basic-link internal-destination="{$id}">
        <xsl:if test="$label != ''">
          <xsl:copy-of select="$label"/>
          <xsl:value-of select="$autotoc.label.separator"/>
        </xsl:if>
        <xsl:apply-templates select="." mode="titleabbrev.markup"/>
      </fo:basic-link>
    </fo:inline>
    <fo:inline keep-together.within-line="always">
      <xsl:text> </xsl:text>
      <!--
      <fo:leader leader-pattern="dots"
                 leader-pattern-width="3pt"
                 leader-alignment="reference-area"
                 keep-with-next.within-line="always"
                 xfc:tab-position="-({$toc.indent.width}pt+7.5mm)"/>
      -->
      <fo:leader leader-pattern="dots"
                 leader-pattern-width="3pt"
                 leader-alignment="reference-area"
                 keep-with-next.within-line="always"/>
      <xsl:text> </xsl:text> 
      <fo:basic-link internal-destination="{$id}">
        <fo:page-number-citation ref-id="{$id}"/>
      </fo:basic-link>
    </fo:inline>
  </fo:block>
</xsl:template>

<!-- ============== Replaces stuff from fo/inline.xsl. =========== -->

<!-- Gets rid of the automatic TM symbol in <productname>. -->
<xsl:template match="productname">
  <xsl:call-template name="inline.charseq"/>
  <!--
  <xsl:if test="@class">
    <xsl:call-template name="dingbat">
      <xsl:with-param name="dingbat" select="@class"/>
    </xsl:call-template>
  </xsl:if>
  -->
</xsl:template>

<!-- ============ Replaces stuff from fo/pagesetup.xsl. =========== -->

<!-- Half-witted way to get rid of the headers on the title page. -->
<xsl:template name="header.table">
  <xsl:param name="pageclass" select="''"/>
  <xsl:param name="sequence" select="''"/>
  <xsl:param name="gentext-key" select="''"/>

  <!-- default is a single table style for all headers -->
  <!-- Customize it for different page classes or sequence location -->

  <xsl:choose>
      <xsl:when test="$pageclass = 'index'">
          <xsl:attribute name="margin-left">0pt</xsl:attribute>
      </xsl:when>
  </xsl:choose>

  <xsl:variable name="column1">
    <xsl:choose>
      <xsl:when test="$double.sided = 0">1</xsl:when>
      <xsl:when test="$sequence = 'first' or $sequence = 'odd'">1</xsl:when>
      <xsl:otherwise>3</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="column3">
    <xsl:choose>
      <xsl:when test="$double.sided = 0">3</xsl:when>
      <xsl:when test="$sequence = 'first' or $sequence = 'odd'">3</xsl:when>
      <xsl:otherwise>1</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="candidate">
    <fo:table table-layout="fixed" width="100%">
      <xsl:call-template name="head.sep.rule">
        <xsl:with-param name="pageclass" select="$pageclass"/>
        <xsl:with-param name="sequence" select="$sequence"/>
        <xsl:with-param name="gentext-key" select="$gentext-key"/>
      </xsl:call-template>

      <fo:table-column column-number="1">
        <xsl:attribute name="column-width">
          <xsl:text>proportional-column-width(</xsl:text>
          <xsl:call-template name="header.footer.width">
            <xsl:with-param name="location">header</xsl:with-param>
            <xsl:with-param name="position" select="$column1"/>
          </xsl:call-template>
          <xsl:text>)</xsl:text>
        </xsl:attribute>
      </fo:table-column>
      <fo:table-column column-number="2">
        <xsl:attribute name="column-width">
          <xsl:text>proportional-column-width(</xsl:text>
          <xsl:call-template name="header.footer.width">
            <xsl:with-param name="location">header</xsl:with-param>
            <xsl:with-param name="position" select="2"/>
          </xsl:call-template>
          <xsl:text>)</xsl:text>
        </xsl:attribute>
      </fo:table-column>
      <fo:table-column column-number="3">
        <xsl:attribute name="column-width">
          <xsl:text>proportional-column-width(</xsl:text>
          <xsl:call-template name="header.footer.width">
            <xsl:with-param name="location">header</xsl:with-param>
            <xsl:with-param name="position" select="$column3"/>
          </xsl:call-template>
          <xsl:text>)</xsl:text>
        </xsl:attribute>
      </fo:table-column>

      <fo:table-body>
        <fo:table-row>
          <xsl:attribute name="block-progression-dimension.minimum">
            <xsl:value-of select="$header.table.height"/>
          </xsl:attribute>
          <fo:table-cell text-align="left"
                         display-align="center">
            <xsl:if test="$fop.extensions = 0">
              <xsl:attribute name="relative-align">baseline</xsl:attribute>
            </xsl:if>
            <fo:block>
              <xsl:call-template name="header.content">
                <xsl:with-param name="pageclass" select="$pageclass"/>
                <xsl:with-param name="sequence" select="$sequence"/>
                <xsl:with-param name="position" select="'left'"/>
                <xsl:with-param name="gentext-key" select="$gentext-key"/>
              </xsl:call-template>
            </fo:block>
          </fo:table-cell>
          <fo:table-cell text-align="center"
                         display-align="center">
            <xsl:if test="$fop.extensions = 0">
              <xsl:attribute name="relative-align">baseline</xsl:attribute>
            </xsl:if>
            <fo:block>
              <xsl:call-template name="header.content">
                <xsl:with-param name="pageclass" select="$pageclass"/>
                <xsl:with-param name="sequence" select="$sequence"/>
                <xsl:with-param name="position" select="'center'"/>
                <xsl:with-param name="gentext-key" select="$gentext-key"/>
              </xsl:call-template>
            </fo:block>
          </fo:table-cell>
          <fo:table-cell text-align="right"
                         display-align="center">
            <xsl:if test="$fop.extensions = 0">
              <xsl:attribute name="relative-align">baseline</xsl:attribute>
            </xsl:if>
            <fo:block>
              <xsl:call-template name="header.content">
                <xsl:with-param name="pageclass" select="$pageclass"/>
                <xsl:with-param name="sequence" select="$sequence"/>
                <xsl:with-param name="position" select="'right'"/>
                <xsl:with-param name="gentext-key" select="$gentext-key"/>
              </xsl:call-template>
            </fo:block>
          </fo:table-cell>
        </fo:table-row>
      </fo:table-body>
    </fo:table>
  </xsl:variable>

  <!-- Really output a header? -->
  <xsl:choose>
    <xsl:when test="$gentext-key='article' and $sequence='first'">
      <!-- This is not the best rule, since it doesn't include the
           table of contents, but I'm not sure what to do about that.
           Note that $pageclass doesn't seem to get set properly
           for 'titlepage' or 'front' for articles. -->
    </xsl:when>
    <xsl:when test="$pageclass = 'titlepage' and $gentext-key = 'book'
                    and $sequence='first'">
      <!-- no, book titlepages have no headers at all -->
    </xsl:when>
    <xsl:when test="$sequence = 'blank' and $headers.on.blank.pages = 0">
      <!-- no output -->
    </xsl:when>
    <xsl:otherwise>
      <xsl:copy-of select="$candidate"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- Half-witted way to get rid of the footers on the title page. -->
<xsl:template name="footer.table">
  <xsl:param name="pageclass" select="''"/>
  <xsl:param name="sequence" select="''"/>
  <xsl:param name="gentext-key" select="''"/>

  <!-- default is a single table style for all footers -->
  <!-- Customize it for different page classes or sequence location -->

  <xsl:choose>
      <xsl:when test="$pageclass = 'index'">
          <xsl:attribute name="margin-left">0pt</xsl:attribute>
      </xsl:when>
  </xsl:choose>

  <xsl:variable name="column1">
    <xsl:choose>
      <xsl:when test="$double.sided = 0">1</xsl:when>
      <xsl:when test="$sequence = 'first' or $sequence = 'odd'">1</xsl:when>
      <xsl:otherwise>3</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="column3">
    <xsl:choose>
      <xsl:when test="$double.sided = 0">3</xsl:when>
      <xsl:when test="$sequence = 'first' or $sequence = 'odd'">3</xsl:when>
      <xsl:otherwise>1</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="candidate">
    <fo:table table-layout="fixed" width="100%">
      <xsl:call-template name="foot.sep.rule">
        <xsl:with-param name="pageclass" select="$pageclass"/>
        <xsl:with-param name="sequence" select="$sequence"/>
        <xsl:with-param name="gentext-key" select="$gentext-key"/>
      </xsl:call-template>
      <fo:table-column column-number="1">
        <xsl:attribute name="column-width">
          <xsl:text>proportional-column-width(</xsl:text>
          <xsl:call-template name="header.footer.width">
            <xsl:with-param name="location">footer</xsl:with-param>
            <xsl:with-param name="position" select="$column1"/>
          </xsl:call-template>
          <xsl:text>)</xsl:text>
        </xsl:attribute>
      </fo:table-column>
      <fo:table-column column-number="2">
        <xsl:attribute name="column-width">
          <xsl:text>proportional-column-width(</xsl:text>
          <xsl:call-template name="header.footer.width">
            <xsl:with-param name="location">footer</xsl:with-param>
            <xsl:with-param name="position" select="2"/>
          </xsl:call-template>
          <xsl:text>)</xsl:text>
        </xsl:attribute>
      </fo:table-column>
      <fo:table-column column-number="3">
        <xsl:attribute name="column-width">
          <xsl:text>proportional-column-width(</xsl:text>
          <xsl:call-template name="header.footer.width">
            <xsl:with-param name="location">footer</xsl:with-param>
            <xsl:with-param name="position" select="$column3"/>
          </xsl:call-template>
          <xsl:text>)</xsl:text>
        </xsl:attribute>
      </fo:table-column>

      <fo:table-body>
        <fo:table-row>
          <xsl:attribute name="block-progression-dimension.minimum">
            <xsl:value-of select="$footer.table.height"/>
          </xsl:attribute>
          <fo:table-cell text-align="left"
                         display-align="after">
            <xsl:if test="$fop.extensions = 0">
              <xsl:attribute name="relative-align">baseline</xsl:attribute>
            </xsl:if>
            <fo:block>
              <xsl:call-template name="footer.content">
                <xsl:with-param name="pageclass" select="$pageclass"/>
                <xsl:with-param name="sequence" select="$sequence"/>
                <xsl:with-param name="position" select="'left'"/>
                <xsl:with-param name="gentext-key" select="$gentext-key"/>
              </xsl:call-template>
            </fo:block>
          </fo:table-cell>
          <fo:table-cell text-align="center"
                         display-align="after">
            <xsl:if test="$fop.extensions = 0">
              <xsl:attribute name="relative-align">baseline</xsl:attribute>
            </xsl:if>
            <fo:block>
              <xsl:call-template name="footer.content">
                <xsl:with-param name="pageclass" select="$pageclass"/>
                <xsl:with-param name="sequence" select="$sequence"/>
                <xsl:with-param name="position" select="'center'"/>
                <xsl:with-param name="gentext-key" select="$gentext-key"/>
              </xsl:call-template>
            </fo:block>
          </fo:table-cell>
          <fo:table-cell text-align="right"
                         display-align="after">
            <xsl:if test="$fop.extensions = 0">
              <xsl:attribute name="relative-align">baseline</xsl:attribute>
            </xsl:if>
            <fo:block>
              <xsl:call-template name="footer.content">
                <xsl:with-param name="pageclass" select="$pageclass"/>
                <xsl:with-param name="sequence" select="$sequence"/>
                <xsl:with-param name="position" select="'right'"/>
                <xsl:with-param name="gentext-key" select="$gentext-key"/>
              </xsl:call-template>
            </fo:block>
          </fo:table-cell>
        </fo:table-row>
      </fo:table-body>
    </fo:table>
  </xsl:variable>

  <!-- Really output a footer? -->
  <xsl:choose>
    <xsl:when test="$gentext-key='article' and $sequence='first'">
      <!-- This is not the best rule, since it doesn't include the
           table of contents, but I'm not sure what to do about that.
           Note that $pageclass doesn't seem to get set properly
           for 'titlepage' or 'front' for articles. -->
    </xsl:when>
    <xsl:when test="$pageclass='titlepage' and $gentext-key='book'
                    and $sequence='first'">
      <!-- No, book title pages have no footer. -->
    </xsl:when>
    <xsl:when test="$sequence = 'blank' and $footers.on.blank.pages = 0">
      <!-- no output -->
    </xsl:when>
    <xsl:otherwise>
      <xsl:copy-of select="$candidate"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- ============== Replaces stuff from fo/params.xsl. =========== -->

<!-- Eliminates a warning message from the converter. -->
<xsl:param name="draft.watermark.image" select="''"/>

<!-- Self-evident, I think -->
<xsl:param name="graphic.default.extension" select="'jpg'"/>

<!-- Adjust the field-sizes in the page-headers to reduce the
     chance that the document name will word-wrap -->
<xsl:param name="header.column.widths" select="'1 10 1'"/>

<!-- Make sure that section numbers are automatically added. -->
<xsl:param name="section.autolabel" select="1"/>

<!-- Set the max section depth in the table of contents. -->
<xsl:param name="toc.max.depth">10</xsl:param>

<!-- ====== Replaces stuff from fo/titlepage.templates.xsl. ======= -->

<!-- Alter the elements included on the title page to include the
     corpname and graphic. -->
<xsl:template name="article.titlepage.recto">
    
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="articleinfo/corpname"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="artheader/corpname"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="info/corpname"/>
    
  <xsl:choose>
    <xsl:when test="articleinfo/title">
      <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="articleinfo/title"/>
    </xsl:when>
    <xsl:when test="artheader/title">
      <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="artheader/title"/>
    </xsl:when>
    <xsl:when test="info/title">
      <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="info/title"/>
    </xsl:when>
    <xsl:when test="title">
      <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="title"/>
    </xsl:when>
  </xsl:choose>

  <xsl:choose>
    <xsl:when test="articleinfo/subtitle">
      <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="articleinfo/subtitle"/>
    </xsl:when>
    <xsl:when test="artheader/subtitle">
      <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="artheader/subtitle"/>
    </xsl:when>
    <xsl:when test="info/subtitle">
      <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="info/subtitle"/>
    </xsl:when>
    <xsl:when test="subtitle">
      <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="subtitle"/>
    </xsl:when>
  </xsl:choose>

  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="articleinfo/corpauthor"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="artheader/corpauthor"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="info/corpauthor"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="articleinfo/authorgroup"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="artheader/authorgroup"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="info/authorgroup"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="articleinfo/author"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="artheader/author"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="info/author"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="articleinfo/othercredit"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="artheader/othercredit"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="info/othercredit"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="articleinfo/releaseinfo"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="artheader/releaseinfo"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="info/releaseinfo"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="articleinfo/copyright"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="artheader/copyright"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="info/copyright"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="articleinfo/legalnotice"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="artheader/legalnotice"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="info/legalnotice"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="articleinfo/graphic"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="artheader/graphic"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="info/graphic"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="articleinfo/pubdate"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="artheader/pubdate"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="info/pubdate"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="articleinfo/revision"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="artheader/revision"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="info/revision"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="articleinfo/revhistory"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="artheader/revhistory"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="info/revhistory"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="articleinfo/abstract"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="artheader/abstract"/>
  <xsl:apply-templates mode="article.titlepage.recto.auto.mode" select="info/abstract"/>
</xsl:template>

<!-- Make sure a page break occurs after the title page. -->
<xsl:template name="article.titlepage.separator"><fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format" break-after="page"/>
</xsl:template>

<!-- Code to display the 'graphic' element on the title page. -->
<xsl:template match="graphic" mode="article.titlepage.recto.auto.mode">
<fo:block xmlns:fo="http://www.w3.org/1999/XSL/Format" xsl:use-attribute-sets="article.titlepage.recto.style" text-align="start" margin-left="0.5in" margin-right="0.5in" font-family="{$body.fontset}">
<xsl:apply-templates select="." mode="article.titlepage.recto.mode"/>
</fo:block>
</xsl:template>

<!-- /////////////////////////////////////////////////////////// -->
<!-- 
  Here's some stuff which has nothing to do with Do178Builder
  per se, but instead gives some neat customizations I've 
  found on the web.  See 
       http://www.dpawson.co.uk/docbook/styling/fo.html
  The customizations include
       <?hard-pagebreak?>
       <long-ulink>
       <long-string>
          
  Inserting <docbook><?hard-pagebreak?></docbook> into your
  Do178Builder file causes a pagebreak to occur at that point.
  This is contrary to the DocBook philosophy, but is useful
  on occasion.  This has to occur at a point where it makes
  sense ... not in the middle of a paragraph, for example.
  There are some very useful places where it *won't* work, 
  such as between items in an ordered or itemized list.
  
  <long-string> could be used to allow line-breaks for long 
  strings not containing appropriate white-space for automatic
  hyphenation, such as a long filename.  It's used like this:
       <long-string str="AnyLongString"/>
  Yes, I think that <long-string>AnyLongString</long-string>
  would be cooler, because you can't add any DocBook markup 
  *within* the long string this way, but I don't know how to do 
  it and am too lazy to figure it out.  <long-string> works by
  placing zero-width spaces between every character of the long
  string, thus providing natural line-break points.
  
  <long-ulink> differs from the normal <ulink> only in 
  that it processes the URL with <long-string>, and in that
  it colors the hyperlink blue, which <ulink> does not.  So if 
  you have a really long URL, it can break into separate lines 
  without messing up the formatting of everything else on the 
  line.  As for why we don't simply *replace* <ulink>, rather 
  than having both <ulink> and <long-ulink>, I don't think you 
  want this behavior all the time.  Note that this applies *only*
  to the URL, and not to the link name.  For example, in 
       <long-ulink url="BigURL">BigLinkName</long-ulink>
  which <ulink> wants to style as
       BigLinkName [BigURL]
  insertion of the zero-width spaces (and hence the additional
  line-break points) applies only to BigURL and not to BigLinkName.  
  If the link name was really big, you could do something like
       <long-ulink url="BigURL">
         <long-string str="BigLinkName"/>
       </long-ulink>
  or       
       <ulink url="BigURL">
         <long-string str="BigLinkName"/>
       </ulink>      
-->

<!-- <?hard-pagebreak?> inserts a page break at this point -->
<xsl:template match="processing-instruction('hard-pagebreak')">
  <fo:block break-before='page'/>
</xsl:template>

<!-- <long-ulink> works like <ulink>, but allows URLs to 
     break across printed lines. -->
<xsl:template match="long-ulink">
  <fo:basic-link external-destination="{@url}"
                 xsl:use-attribute-sets="xref.properties"
                 color="#0000AA">
    <xsl:choose>
      <xsl:when test="count(child::node())=0">
        <xsl:value-of select="@url" />
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates />
      </xsl:otherwise>
    </xsl:choose>
  </fo:basic-link>
  <xsl:if test="count(child::node()) != 0">
    <fo:inline hyphenate="true">
      <xsl:text> [</xsl:text>
      <!--<xsl:value-of select="@url"/>-->
      <xsl:call-template name="intersperse-spaces">
        <xsl:with-param name="str" select="@url"/>
      </xsl:call-template>
      <xsl:text>]</xsl:text>
    </fo:inline>
  </xsl:if>
</xsl:template>

<xsl:template match="long-string">
  <fo:inline hyphenate="true">
    <xsl:call-template name="intersperse-spaces">
      <xsl:with-param name="str" select="@str"/>
    </xsl:call-template>
  </fo:inline>
</xsl:template>

<!-- Used by <long-ulink> and <long-string> above.
     Actual space intercalation: recursive -->
<xsl:template name="intersperse-spaces">
  <xsl:param name="str"/>
  <!-- Various Unicode space characters. -->
  <xsl:variable name="spacechars">
    &#x9;&#xA;&#x2000;&#x2001;&#x2002;&#x2003;&#x2004;&#x2005;
    &#x2006;&#x2007;&#x2008;&#x2009;&#x200A;&#x200B;
  </xsl:variable>

  <xsl:if test="string-length($str) &gt; 0">
    <xsl:variable name="c1" select="substring($str, 1, 1)"/>
    <xsl:variable name="c2" select="substring($str, 2, 1)"/>

    <xsl:value-of select="$c1"/>
    <xsl:if test="$c2 != '' and
      not(contains($spacechars, $c1) or
      contains($spacechars, $c2))">
      <!-- "&#x200A;" is a "&hairsp;", which is pretty thin.
           "&#x200B;" would be a zero-width space, which is even 
           thinner, but apparently doesn't work for some XSL
           processors.  Also, I personally think that using the
           hairline space makes the line-break look a little
           less unnatural than the zero-width space does. 
           So there are pros and cons for each. -->
      <xsl:text>&#x200B;</xsl:text>
    </xsl:if>
    <xsl:call-template name="intersperse-spaces">
      <xsl:with-param name="str" select="substring($str, 2)"/>
    </xsl:call-template>
  </xsl:if>
</xsl:template>

<!-- /////////////////////////////////////////////////////////// -->

</xsl:stylesheet> 
 
